package Data::Filter::Rule;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Data::Filter::Rule - Base class for all data filter rule tree nodes

=head1 SYNOPSIS

 use Data::Filter::Rule;

 my $filter_rule1 = Data::Filter::Rule->new_rule('const', 'const_value');
 my $filter_rule2 = Data::Filter::Rule->new_rule('var', 'None/Name');
 my $filter_rule3 = Data::Filter::Rule->new_rule('eq', $filter2, $filter1);

=head1 DESCRIPTION



=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.


=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';

#use Data::Dumper; #--- DEVELOPMENT ONLY ---#
#use Smart::Comments; #--- DEVELOPMENT ONLY ---#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL);
    $VERSION = '0.1';
    $DEVEL   = 0;
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

# Operator overloading
use overload '""' => 'to_string';

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item new [PUBLIC,STATIC]

  Usage     : my $instance = Data::Filter::Rule->new()
  Purpose   : Create and return reference to the new instance
  Arguments : Same as the the _init() method
  Returns   :
  Throws    : Croaks, if invoked on object
  Comments  : Internally uses _init() method
  See Also  : _init() method

=cut

sub new
{
    my $class = shift;
    confess "Class method not invoked on class" if blessed($class);
    my $self = bless ({}, $class);
    return $self->_init(@_);
}

=item name() [PUBLIC, ABSTRACT]

  Usage     : my $name = $rule->name();
  Purpose   :
  Returns   :
  Arguments :
  Throws    :
  Comments  :
  See Also  :

=cut

sub name()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    # Determine the name of the rule (last part of the class name)
    unless ($self->{NAME}) {
        my $name = blessed($self);
        my @tmp = split(/:+/, $name);
        $self->{NAME} = $tmp[$#tmp];
    }
    return $self->{NAME};
}

=item datatype() [PUBLIC, ABSTRACT]

  Usage     : my $result = $rule->datatype();
  Purpose   :
  Returns   :
  Arguments :
  Throws    :
  Comments  :
  See Also  :

=cut

sub datatype()
{
    my $self = shift;
    confess "Method needs implementation";
}

=item value() [PUBLIC, ABSTRACT]

  Usage     : my $result = $rule->value();
  Purpose   :
  Returns   :
  Arguments :
  Throws    :
  Comments  :
  See Also  :

=cut

sub value()
{
    my $self = shift;
    confess "Method needs implementation";
}

=item operation() [PUBLIC, ABSTRACT]

  Usage     : my $result = $rule->operation();
  Purpose   :
  Returns   :
  Arguments :
  Throws    :
  Comments  :
  See Also  :

=cut

sub operation()
{
    my $self = shift;
    confess "Method needs implementation";
}

#-------------------------------------------------------------------------------

=item traverse($) [PUBLIC, ABSTRACT]

  Usage     : my $result = Data::Filter::Rule->traverse($traverser);
  Purpose   : Traversal through the filter rule tree with given traverser object
  Returns   :
  Arguments :
  Throws    :
  Comments  :
  See Also  :

=cut

sub traverse($)
{
    my $self = shift;
    confess "Method needs implementation";
}

#-------------------------------------------------------------------------------

=item to_string() [PUBLIC, ABSTRACT]

  Usage     : my $result = Data::Filter::Rule->to_string();
  Purpose   :
  Returns   :
  Arguments :
  Throws    :
  Comments  :
  See Also  :

=cut

sub to_string()
{
    my $self = shift;
    confess "Method needs implementation";
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init($) [PROTECTED,ABSTRACT]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  :
#  See Also  :

sub _init($)
{
    my $self = shift;
    confess "Method needs implementation";
}

=pod

=back

=cut

1;
