package Data::Query::Object;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Data::Query::Object - Execute query on objects and Perl data structures

=head1 SYNOPSIS

 use Data::Query::Object;

 # Instantinate the data query object
 my $query = Data::Query::Object->new(dt_resolver => Data::Path::Value->new(...));

 # Check, if the given object passes through given filter (additional method to the interface)
 my $result = $query->is_satisfied_by($filter, $object)

 # Filter out the objects which do not match the filter
 my $objects = [$object1, $object2, $object3, $object4];
 my $passed = $query->query($filter, $objects);

=head1 DESCRIPTION

This class was designed to be used for filtering arbitrary Data::Tree and Perl
data structures.

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1), Data::Filter::Traverser(3), Data::Query(3), Data::Tree(3).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';

use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #--- DEVELOPMENT ONLY ---#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#
use Data::Filter::Traverser;
use Data::Query;
use Data::Tree;

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL @ISA);
    $VERSION = '0.1';
    $DEVEL   = 0;
    @ISA     = ('Data::Filter::Traverser', 'Data::Query');
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item is_satisfied_by($$) [PUBLIC]

  Usage     : my $result = $instance->is_satisfied_by($filter, $object)
  Purpose   : Check, if the given object satisfies the given filter
  Arguments : Data::Filter $filter - query filter tree
              (Data::Tree|HASH REFERENCE) $object - object to check
  Returns   : INTEGER 1, if object satisfies the filter, INTEGER 0 otherwise
  Throws    : Dies, if not invoked on object

=cut

sub is_satisfied_by($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($filter, $object) = @_;
    confess "Queried object is expected to be given as argument" unless $object;
    confess "Queried object must be blessed object or hash reference" unless ref $object eq 'HASH' or blessed($object);

    $self->_current_object($object);
    my $result = $filter->traverse($self);

    delete $self->{OBJECT};
    return $result;
}

=item query($;@) [PUBLIC]

  Usage     : my $passed = $instance->query($filter, $objects)
  Purpose   : Apply the filter on given objects
  Arguments : Data::Filter $filter - query filter tree
              ARRAY REFERENCE $objects - list containing multiple object references
  Returns   : ARRAY REFERENCE containing passed objects
  Throws    : Croaks, if not invoked on object

=cut

sub query($;@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($filter, $objects) = @_;

    my $result = [];
    foreach my $obj (@$objects) {
        push(@$result, $obj) if $self->is_satisfied_by($filter, $obj);
    }
    return $result;
}

#-------------------------------------------------------------------------------

=item FILTER CALLBACKS [PUBLIC]

  Usage     : return $traverser->$name(@args);
  Purpose   : Rule tree traversal callbacks, for each rule class one callback method
  Arguments : Depend on particular callback
  Returns   : Any result from callback will be passed to the parent node callback as argument
  Throws    : Croaks, if invoked on object
  Comments  : These methods are called from Data::Filter::traverse method
              during filter tree traversal. Arguments are dependent on

=cut

sub constant($;@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($filter, @args) = @_;

    return $filter->value_raw();
}
sub list($;@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($filter, @args) = @_;

    return $filter->value_raw();
}
sub existence($;@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($filter, @args) = @_;

    return $self->_current_object->path_node_exists($filter->value());
}
sub variable($;@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($filter, @args) = @_;

    return $self->_current_object->path_values($filter->value());
}
# ---
sub operationUnary($$$$;@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($filter, $operation, $subfilter, $operand, @args) = @_;

    return $self->evaluate_n_unr($operation, $operand);
}
sub operationBinary($$$$$$;@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($filter, $operation, $l_subfilter, $r_subfilter, $l_operand, $r_operand, @args) = @_;

    my $evaluator = $self->pick_evaluator($operation, $l_subfilter, $r_subfilter);

    print STDERR "B----------B\n" if $DEVEL;
    print STDERR "EVALUATING:\n" if $DEVEL;
    print STDERR Dumper($evaluator, $l_operand, $r_operand) if $DEVEL;

    my $r = $self->evaluate_n_bin($evaluator, $l_operand, $r_operand);
    print STDERR " --- \n" if $DEVEL;
    print STDERR Dumper($r) if $DEVEL;
    print STDERR "E----------E\n" if $DEVEL;
    return $r;
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Data::Query::Object instance
#  Arguments :
#  Returns   : Data::Query::Object reference
#  Throws    : Croaks, if invoked on class
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.
#  See Also  :

sub _init
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my %args = @_;
    return $self;
}

# _current_object(;$) [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   : Data::Query::Object reference
#  Throws    : Croaks, if invoked on class
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.

sub _current_object(;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($object,) = @_;

    # Setter mode
    if ($object) {
        $self->{OBJECT} = (ref($object) eq 'HASH') ? Data::Tree->new($object) : $object;
    }
    return $self->{OBJECT};
}

=pod

=back

=cut

1;
