package Hawat::Module::System;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Hawat::Module::System

=head1 SYNOPSIS

 use Hawat::Module::System;

=head1 DESCRIPTION



=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';
use Time::HiRes qw (time);
use File::Find;

#use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#
use Hawat::Module;
use Mentat::System;

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#
use constant SPOOL_THRESHOLD => 2000;

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

# Hash containing all valid commands supported by this module and their asociated
# callback names for indirect execution.
my %COMMANDS = (
            'spool' => {
                    '_DESC_' => 'Manage the Mentat spool directory',
                    ''      => [
                            '_command_spool_list' ,
                            'View the status of Mentat spool directory',
                        ],
                    'list'  => [
                            '_command_spool_list' ,
                            'View the status of Mentat spool directory',
                        ],
                    'clean' => [
                            '_command_spool_clean' ,
                            'Cleanup Mentat spool directory',
                        ]
                },
            'pidfile'   => {
                    '_DESC_' => 'Manage the PID files for running Mentat components',
                    ''      => [
                            '_command_pidfile_list' ,
                            'List the PID files for running Mentat components',
                        ],
                    'list'  => [
                            '_command_pidfile_list' ,
                            'List the PID files for running Mentat components',
                        ],
                    'clean' => [
                            '_command_pidfile_clean' ,
                            'Cleanup dangling PID files for Mentat components',
                        ]
                },
        );

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL @ISA);
    $VERSION = '0.1';
    $DEVEL   = 0;
    @ISA     = ('Hawat::Module');
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item get_commands() [PUBLIC]

  Usage     :
  Purpose   : Return list of all commands this module provides
  Arguments :
  Returns   :
  Throws    : Croaks, if invoked on class
  Comments  :
  See Also  :

=cut

sub get_commands()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    return %COMMANDS;
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Hawat::Module::System instance
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.
#  See Also  :

sub _init
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    $self->{SPOOL_FOLDER} = $self->cfg->{'spool_folder'};
    $self->{RUN_FOLDER}   = $self->cfg->{'run_folder'};

    return $self;
}

#-------------------------------------------------------------------------------

# _spool_list() [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  :
#  See Also  :

sub _spool_list()
{
    my $self    = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    my $start = time();

    my $spool = Mentat::System->spool_list($self->{SPOOL_FOLDER});

    my @modules = sort keys(%$spool);

    $self->res->{'count'} = scalar(@modules);
    my $cnt;
    foreach my $module (@modules) {
        $self->out("   - $module:\n");

        foreach my $dir (sort keys(%{$spool->{$module}})) {
            $cnt = $spool->{$module}->{$dir};
            $self->out("      - $dir: $cnt");
            if ($cnt <= SPOOL_THRESHOLD) {
                $self->out(" [OK]\n", 'bold green');
                push(@{$self->res->{'success'}}, "$module:$dir:$cnt");
            }
            else {
                $self->out(" [ERROR]\n", 'bold red');
                push(@{$self->res->{'failure'}}, "$module:$dir:$cnt");
            }
        }
    }

    my $stop = time();
    $self->out(" - Spool list duration: " . ($stop - $start) . " [s]\n");
    $self->res->{'duration'} = $stop - $start;
}

# _spool_clean() [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  :
#  See Also  :

sub _spool_clean()
{
    my $self    = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    my $start = time();

    my $spool = Mentat::System->spool_clean($self->{SPOOL_FOLDER});

    my @modules = sort keys(%$spool);

    $self->res->{'count'} = scalar(@modules);
    my $cnt;
    foreach my $module (@modules) {
        $self->out("   - $module:\n");

        foreach my $r (sort keys(%{$spool->{$module}})) {
            $cnt = $spool->{$module}->{$r};
            $self->out("      - $r: $cnt");
            if ($r eq 'removed') {
                $self->out(" [OK]\n", 'bold green');
                push(@{$self->res->{'success'}}, "$module:$r:$cnt");
            }
            elsif ($cnt == 0) {
                $self->out(" [OK]\n", 'bold green');
            }
            else {
                $self->out(" [ERROR]\n", 'bold red');
                push(@{$self->res->{'failure'}}, "$module:$r:$cnt");
            }
        }
    }

    my $stop = time();
    $self->out(" - Spool cleanup duration: " . ($stop - $start) . " [s]\n");
    $self->res->{'duration'} = $stop - $start;
}

#-------------------------------------------------------------------------------

# _pidfile_list() [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  :
#  See Also  :

sub _pidfile_list()
{
    my $self    = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    my $start = time();

    my $pidfiles = Mentat::System->pidfile_list($self->{RUN_FOLDER});

    my @modules = sort keys(%$pidfiles);

    $self->res->{'count'} = scalar(@modules);
    my $cnt;
    foreach my $module (@modules) {
        $self->out("   * $module:\n");
        $self->out("       - file:      ".$pidfiles->{$module}->{file}."\n");
        $self->out("       - pid:       ".$pidfiles->{$module}->{pid}."\n");
        $self->out("       - running:   ".$pidfiles->{$module}->{running});
        if ($pidfiles->{$module}->{running}) {
            $self->out(" [OK]\n", 'bold green');
        }
        else {
            $self->out(" [ERROR]\n", 'bold red');
        }
        $self->out("       - running_0: ".$pidfiles->{$module}->{running_0});
        if ($pidfiles->{$module}->{running_0}) {
            $self->out(" [OK]\n", 'bold green');
        }
        else {
            $self->out(" [ERROR]\n", 'bold red');
        }
    }

    my $stop = time();
    $self->out(" - Spool list duration: " . ($stop - $start) . " [s]\n");
    $self->res->{'duration'} = $stop - $start;
}

# _pidfile_clean() [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  :
#  See Also  :

sub _pidfile_clean()
{
    my $self    = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    my $start = time();

    my $result = Mentat::System->pidfile_clean($self->{RUN_FOLDER});

    my @modules = sort keys(%$result);

    $self->res->{'count'} = scalar(@modules);
    my $cnt;
    foreach my $module (@modules) {
        $self->out("   * $module:\n");
        foreach my $r (sort @{$result->{$module}}) {
            $self->out("      - $r\n");
        }
    }

    my $stop = time();
    $self->out(" - Spool cleanup duration: " . ($stop - $start) . " [s]\n");
    $self->res->{'duration'} = $stop - $start;
}

#-----------------------------------------------------------------------

# _command_spool_list [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  :
#  See Also  :

sub _command_spool_list
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($chain, @args) = @_;
    die "Invalid usage: [$0 @$chain] command does not accept any arguments\n" if @args;

    $self->out(" - Status of Mentat spool directory:\n");
    $self->_spool_list();
}

# _command_spool_clean [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  :
#  See Also  :

sub _command_spool_clean
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($chain, @args) = @_;
    die "Invalid usage: [$0 @$chain] command does not accept any arguments\n" if @args;

    $self->out(" - Cleaning Mentat spool directory:\n");
    $self->_spool_clean();
}

#-----------------------------------------------------------------------

# _command_pidfile_list [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  :
#  See Also  :

sub _command_pidfile_list
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($chain, @args) = @_;
    die "Invalid usage: [$0 @$chain] command does not accept any arguments\n" if @args;

    $self->out(" - Status of Mentat component PID files:\n");
    $self->_pidfile_list();
}

# _command_pidfile_clean [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  :
#  See Also  :

sub _command_pidfile_clean
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($chain, @args) = @_;
    die "Invalid usage: [$0 @$chain] command does not accept any arguments\n" if @args;

    $self->out(" - Cleaning dangling Mentat PID files:\n");
    $self->_pidfile_clean();
}

=pod

=back

=cut

1;
