package Mentat::Core;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Mentat::Core - String related utilities

=head1 SYNOPSIS

 use Mentat::Core qw(const);

=head1 DESCRIPTION

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.


=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Cwd qw(abs_path);
use FindBin;
use File::Basename;
use JSON;

#use Data::Dumper; #--- DEVELOPMENT ONLY ---#
#use Smart::Comments; #--- DEVELOPMENT ONLY ---#

#-- Perl CPAN modules ---------------------------------------------------------#
use Bytes::Random::Secure qw(random_string_from);
use Data::Random qw(rand_chars);

#-- Custom application modules ------------------------------------------------#

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-------------------------------------------------------------------------------
# GENERAL CONSTANTS
#-------------------------------------------------------------------------------
use constant TRUE    => 1;
use constant FALSE   => 0;
use constant YES     => 1;
use constant NO      => 0;
use constant SUCCESS => 0;
use constant FAILURE => 1;

use constant ANY     => '__ANY__';

use constant EC_CONTINUE => 1;
use constant EC_STOP     => 0;

use constant ENV_SYSTEM  => '__SYSTEM__';
use constant ENV_PACKAGE => '__PACKAGE__';
use constant ENV_DEVEL_S => '__DEVEL_S__';
use constant ENV_DEVEL   => '__DEVEL__';
use constant ENV_TEST    => '__TEST__';

#-------------------------------------------------------------------------------
# OPERATIONS
#-------------------------------------------------------------------------------
use constant OP_AND => 'AND';
use constant OP_OR  => 'OR';
use constant OP_XOR => 'XOR';

use constant OP_P_AND => '&&';
use constant OP_P_OR  => '||';
use constant OP_P_XOR => '^^';

use constant OP_EQ => 'EQ';
use constant OP_NE => 'NE';
use constant OP_GT => 'GT';
use constant OP_LT => 'LT';
use constant OP_GE => 'GE';
use constant OP_LE => 'LE';

use constant OP_LIKE => 'LIKE';

use constant OP_IN => 'IN';

use constant OP_ADD => 'ADD';
use constant OP_SUB => 'SUB';
use constant OP_MUL => 'MUL';
use constant OP_DIV => 'DIV';
use constant OP_MOD => 'MOD';
use constant OP_CON => 'CON';
use constant OP_REP => 'REP';

#-------------------------------------------------------------------------------
# LOW-LEVEL DATATYPES
#
# !!!! WARNING !!! Some of these datatypes are hardcoded in filter grammar !!!!
#-------------------------------------------------------------------------------
use constant DT_UNKNOWN   => 'UNKNOWN';
use constant DT_DEPENDENT => 'DEPENDENT';
use constant DT_BOOL      => 'BOOL';
use constant DT_BYTE      => 'BYTE';
use constant DT_DATETIME  => 'DATETIME';
use constant DT_ENUM      => 'STRING';
use constant DT_INT       => 'INT';
use constant DT_NTPSTAMP  => 'NTPSTAMP';
use constant DT_PORTLIST  => 'PORTLIST';
use constant DT_REAL      => 'REAL';
use constant DT_STRING    => 'STRING';

#-------------------------------------------------------------------------------
# HIGH-LEVEL DATATYPES
#-------------------------------------------------------------------------------
use constant DT_PERIOD   => 'PERIOD';
use constant DT_NUMERIC  => 'NUMERIC';
use constant DT_ADDRESS  => 'ADDRESS';
use constant DT_IPV4     => 'IPV4';
use constant DT_IPV6     => 'IPV6';
use constant DT_MAC      => 'MAC';
use constant DT_EMAIL    => 'EMAIL';
use constant DT_DURATION => 'DURATION';

use constant DT_IPV4_STR          => 'IPV4-ADDR';
use constant DT_IPV4_HEX          => 'IPV4-ADDR-HEX';
use constant DT_IPV4_STR_NET      => 'IPV4-NET';
use constant DT_IPV4_STR_NET_MASK => 'IPV4-NET-MASK';
use constant DT_IPV6_STR          => 'IPV6-ADDR';
use constant DT_IPV6_HEX          => 'IPV6-ADDR-HEX';
use constant DT_IPV6_STR_NET      => 'IPV6-NET';
use constant DT_IPV6_STR_NET_MASK => 'IPV6-NET-MASK';

use constant DT_REGEXP => 'REGEXP';

#-------------------------------------------------------------------------------
# PROCESSOR AND MAIN FRONT CONTROLLER RELATED CONSTANTS
#-------------------------------------------------------------------------------
use constant CSECTION_LOGER      => '_LOGER_';
use constant CSECTION_PROCESSOR  => '_PROCESSOR_';
use constant CSECTION_SCHEDULER  => '_SCHEDULER_';
use constant CSECTION_COMPONENTS => '_COMPONENTS_';
use constant CSECTION_DAEMON     => '_DAEMON_';

use constant CSECTION_STARTUP  => '_STARTUP_';
use constant CSECTION_LOOP     => '_LOOP_';
use constant CSECTION_SHUTDOWN => '_SHUTDOWN_';

use constant CKEY_LOG      => 'log';
use constant CKEY_PATH     => 'path';
use constant CKEY_SIGNALS  => 'signals';
use constant CKEY_RUNLOG   => 'runlog';
use constant CKEY_CLASS    => 'class';
use constant CKEY_PID_FILE => 'pid_file';
use constant CKEY_USER     => 'user';
use constant CKEY_GROUP    => 'group';
use constant CKEY_INFOREG  => 'in_foreground';
use constant CKEY_EVENTS   => 'events';

use constant STATE_NORMAL => 'NORMAL';
use constant STATE_PAUSED => 'PAUSED';

#-------------------------------------------------------------------------------
# HAWAT RELATED CONSTANTS
#-------------------------------------------------------------------------------

use constant KEY_CACHE_NODENAMES  => "itemset-nodenames";
use constant KEY_CACHE_NODESWS    => "itemset-nodesws";
use constant KEY_CACHE_CATEGORIES => "itemset-categories";
use constant KEY_CACHE_DETECTORS  => "itemset-detectors";
use constant KEY_CACHE_NODECATEG  => "itemset-nodenamecategories";
use constant KEY_CACHE_SWCATEG    => "itemset-nodeswcategories";
use constant KEY_CACHE_EVENTLIB   => "eventlib-index";

use constant HAWAT_MSG_HANDLER     => '/hnd_message';
use constant HAWAT_ERROR_HANDLER   => '/hnd_error';
use constant HAWAT_ERROR_HANDLER_R => '/hnd_error_redir';
use constant HAWAT_DBI_FIND        => '/dbitem/find';
use constant HAWAT_DBI_LIST        => '/dbitem/list';
use constant HAWAT_DBI_FETCH       => '/dbitem/fetch';
use constant HAWAT_DBI_CREATE      => '/dbitem/create';
use constant HAWAT_DBI_UPDATE      => '/dbitem/update';
use constant HAWAT_DBI_RENAME      => '/dbitem/rename';
use constant HAWAT_DBI_MODIFY      => '/dbitem/modify';
use constant HAWAT_DBI_DELETE      => '/dbitem/delete';
use constant HAWAT_DBI_REMOVE      => '/dbitem/remove';
use constant HAWAT_DBI_SCREATE     => '/dbitem/subCreate';
use constant HAWAT_DBI_SUPDATE     => '/dbitem/subUpdate';
use constant HAWAT_DBI_SDELETE     => '/dbitem/subDelete';
use constant HAWAT_DBI_DO          => '/dbitem/do';

use constant HAWAT_ATTR_MID       => '_id';
use constant HAWAT_ATTR_ID        => 'id';
use constant HAWAT_ATTR_NAME      => "name";
use constant HAWAT_ATTR_ORG       => "organization";
use constant HAWAT_ATTR_EMAIL     => "email";
use constant HAWAT_ATTR_ROLES     => "roles";
use constant HAWAT_ATTR_GROUPS    => "groups";
use constant HAWAT_ATTR_ORGGROUPS => "orggroups";
use constant HAWAT_ATTR_AFFIL     => "affiliations";
use constant HAWAT_ATTR_QUERIES   => "query";
use constant HAWAT_ATTR_TS        => "ts";

use constant HAWAT_ATTR_DESC     => "description";
use constant HAWAT_ATTR_SOURCE   => "source";
use constant HAWAT_ATTR_NETWORKS => "networks";
use constant HAWAT_ATTR_MANAGERS => "managers";

use constant HAWAT_MODEL_ACCESSLOGS => "Accesslogs";
use constant HAWAT_MODEL_ALERTS     => "Alerts";
use constant HAWAT_MODEL_CHANGELOGS => "Changelogs";
use constant HAWAT_MODEL_DASHBOARD  => "Dashboard";
use constant HAWAT_MODEL_EVENTS     => "Events";
use constant HAWAT_MODEL_GROUPS     => "Groups";
use constant HAWAT_MODEL_LOGS       => "Logs";
use constant HAWAT_MODEL_PAGINATOR  => "Paginator";
use constant HAWAT_MODEL_QUERY      => "Query";
use constant HAWAT_MODEL_REPORTS    => "Reports";
use constant HAWAT_MODEL_STATISTICS => "Statistics";
use constant HAWAT_MODEL_USERS      => "Users";
use constant HAWAT_MODEL_WHOIS      => "Whois";

use constant HAWAT_VIEW_HTML => "HTML";
use constant HAWAT_VIEW_JSON => "JSON";

use constant HAWAT_FORM_AUTOFILL => "Autofill";
use constant HAWAT_FORM_PREVIEW  => "Preview";
use constant HAWAT_FORM_CANCEL   => "Cancel";

use constant HAWAT_ACCOUNT_ANONYMOUS => "Anonymous";

# TODO: Cleanup the rest of the constants, most of them does not belong here

use constant HAWAT_ID_QUERY               => "id_query";
use constant HAWAT_ID_GROUPS              => "id_groups";

use constant HAWAT_FORM                   => "form";
use constant HAWAT_FORM_ID                => "id";
use constant HAWAT_FORM_SIMPLE            => "simple";
use constant HAWAT_FORM_SAVE              => "save";
use constant HAWAT_FORM_QUERY_NAME        => "name";
use constant HAWAT_FORM_LIMIT             => "limit";
use constant HAWAT_FORM_SKIP              => "skip";
use constant HAWAT_FORM_SORTBY            => "sortby";
use constant HAWAT_FORM_SORT              => "sort";
use constant HAWAT_FORM_FILTER            => "filter";
use constant HAWAT_FORM_IPSRC             => "source";
use constant HAWAT_FORM_IPOR              => "logop";
use constant HAWAT_FORM_IPDST             => "target";
use constant HAWAT_FORM_DATEFROM          => "datefrom";
use constant HAWAT_FORM_DATETO            => "dateto";
use constant HAWAT_FORM_CLASSIFICATION    => "classification";
use constant HAWAT_FORM_SELECT_CLASS      => "select_class";
use constant HAWAT_FORM_DETECTOR          => "detector";
use constant HAWAT_FORM_SELECT_DETECTOR   => "select_detector";
use constant HAWAT_FORM_SELECT_GROUP      => "select_group";
use constant HAWAT_FORM_QUERY             => "query";
use constant HAWAT_FORM_GROUP             => "group";

use constant HAWAT_FORM_PAGE              => "page";
use constant HAWAT_FORM_PAGINATOR         => "paginator";

use constant HAWAT_TABLE_BODY             => "tbody";
use constant HAWAT_TABLE_BODY_USER_QUERY  => "tbody_user_query";
use constant HAWAT_TABLE_BODY_GROUP_QUERY => "tbody_group_query";
use constant HAWAT_TABLE_QUERY            => "query_table";
use constant HAWAT_DURATION               => "duration";
use constant HAWAT_COUNT                  => "count";
use constant HAWAT_COUNT_ALL              => "count_all";
use constant HAWAT_LAST_FILTER            => "last_filter";
use constant HAWAT_LAST_INDEX             => "last_index";

use constant HAWAT_CHECKED                => "checked";

use constant HAWAT_FORWARD_VIEW_HTML      => "View::HTML";
use constant HAWAT_FORWARD_VIEW_JSON      => "View::JSON";

use constant HAWAT_FORM_FROM              => "from";
use constant HAWAT_FORM_TO                => "to";
use constant HAWAT_FORM_INTERVALS         => "intervals";
use constant HAWAT_FORM_DATA              => "data";
use constant HAWAT_FORM_ERROR             => "error";

use constant HAWAT_PATH_SOURCE_IP         => "Source/IP4/ip";
use constant HAWAT_PATH_TARGET_IP         => "Target/IP4/ip";

use constant HAWAT_PAGE_FIRST_PAGE        => "first_page";
use constant HAWAT_PAGE_LAST_PAGE         => "last_page";
use constant HAWAT_PAGE_CURRENT_PAGE      => "current_page";
use constant HAWAT_PAGE_PREVIOUS_PAGE     => "previous_page";
use constant HAWAT_PAGE_NEXT_PAGE         => "next_page";
use constant HAWAT_PAGE_MIN               => "min";
use constant HAWAT_PAGE_MAX               => "max";

use constant HAWAT_QUERY_TITLE            => "title";

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#
my $DFLT_CHARS  = join('', ('a' .. 'z'), ('A' .. 'Z'), ('0' .. '9'));
my $DFLT_LENGTH = 10;

# Detected current environment
my $ENVIRONMENT;
my %LOCATIONS;

#*******************************************************************************
# HELPER METHOD SECTION
#*******************************************************************************

# Runtime environment detection
sub detect_runtime_environment()
{
    return $ENVIRONMENT if $ENVIRONMENT;

    # Configuration location in package installation
    if    ($FindBin::Bin =~ m|^/usr/local/bin|)          { return $ENVIRONMENT = ENV_SYSTEM; }
    # Configuration location in tarball installation
    elsif ($FindBin::Bin =~ m|^/opt/mentat/bin|)         { return $ENVIRONMENT = ENV_PACKAGE; }
    # Local configurations in unit test environment
    elsif ($FindBin::Bin =~ m|tests/unit|)               { return $ENVIRONMENT = ENV_TEST; }
    # Local configurations in server development environment
    elsif ($FindBin::Bin =~ m|^/home/mentat/mentat/bin|) { return $ENVIRONMENT = ENV_DEVEL_S; }
    # Local configurations in development environment
    else                                                 { return $ENVIRONMENT = ENV_DEVEL; }
}

# Set specific runtime environment
sub set_runtime_environment($) {
    $ENVIRONMENT = shift;
    unless ($ENVIRONMENT and exists($LOCATIONS{$ENVIRONMENT})) {
        die "Attempted to set unknown runtime environment '$ENVIRONMENT'\n";
        $ENVIRONMENT = undef;
        detect_runtime_environment();
    }
}

# Environment dependent locations
sub loc_bin() { return (abs_path("$FindBin::Bin") =~ m{^(/.*)$})[0]; }
sub loc_etc() { return $LOCATIONS{$ENVIRONMENT}->{etc}; }
sub loc_cre() { return $LOCATIONS{$ENVIRONMENT}->{cre}; }
sub loc_log() { return $LOCATIONS{$ENVIRONMENT}->{log}; }
sub loc_run() { return $LOCATIONS{$ENVIRONMENT}->{run}; }
sub loc_tmp() { return $LOCATIONS{$ENVIRONMENT}->{tmp}; }
sub loc_var() { return $LOCATIONS{$ENVIRONMENT}->{var}; }

# Application name
sub app_name() { return basename(${0}); }

# Sanitize file path
sub p($)
{
    my $p = shift; my $t = abs_path($p);
    $p = ($p =~ m{^(.*)$})[0] if $p;
    $p =~ s|/\./|/| if $p;
    $t = ($t =~ m{^(.*)$})[0] if $t;
    return ($t)?$t:$p;
}

# Application environment dependent location of configuration file
sub detect_config_file(;$)
{
    my $suffix = shift; $suffix = 'conf' unless $suffix;
    return p(loc_etc()."/".app_name.".$suffix");
}

# Attempt to generate unique ID
sub uid(;$$)
{
    my ($chars, $length) = @_;
    $chars  = $DFLT_CHARS  unless $chars;
    $length = $DFLT_LENGTH unless $length;
    return random_string_from($chars, $length);
}

# Attempt to generate unique ID
sub uid_fast(;$)
{
    my ($length,) = @_;
    $length = $DFLT_LENGTH unless $length;

    my @rnd = rand_chars(set => 'alphanumeric', size => $length);
    return join('', @rnd);
}

#-------------------------------------------------------------------------------

sub json_load
{
    my ($jsonfile,) = @_;

    return {} unless -s $jsonfile;

    #use utf8;

    # Load JSON string from external file
    #open (my $fh, '<:encoding(UTF-8)', $jsonfile) or confess "Can`t open JSON file '$jsonfile' for reading";
    open (my $fh, $jsonfile) or confess "Can`t open JSON file '$jsonfile' for reading";
    my $json_str;
    while (my $line = <$fh>) {
        # Custom feature - remove comments
        next if $line =~ /^\s*(#|;|\/\/)/;
        $json_str .= $line;
    }
    close($fh);

    # Encode utf8 string
    #if (utf8::is_utf8($json_str)) {
    #if (1) {
    #   use Encode;
    #   $json_str = encode_utf8($json_str);
    #}

    # Untaint the loaded string
    if ($json_str =~ /^(.+)$/s) { $json_str = $1; }

    # Parse out the JSON object and return it
    my $data = from_json($json_str, {utf8 => 1});
    return $data ? $data : {};
}

sub json_save
{
    my ($data, $jsonfile) = @_;

    my $json = to_json($data, {utf8 => 1, pretty => 1});

    open(my $hnd, ">$jsonfile") or die "Can`t open JSON file '$jsonfile' for writing\n";
    print $hnd $json."\n";
    close($hnd);
}

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use Exporter ();

    our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);

    $VERSION = 1.00;
    @ISA = qw(Exporter);

    # Autoexport - currently unused
    #@EXPORT = qw(&func &func);

    # Export by tags
    %EXPORT_TAGS = (
        c_general => [qw(TRUE FALSE YES NO SUCCESS FAILURE ANY EC_CONTINUE EC_STOP
                         ENV_SYSTEM ENV_PACKAGE ENV_DEVEL ENV_DEVEL_S ENV_TEST)],
        c_logop => [qw(OP_AND OP_OR OP_XOR OP_EQ OP_NE OP_GT OP_LT OP_GE OP_LE
                       OP_LIKE OP_IN)],
        c_modop => [qw(OP_ADD OP_SUB OP_MUL OP_DIV OP_MOD OP_CON OP_REP)],
        c_opers => [qw(OP_AND OP_OR OP_XOR OP_EQ OP_NE OP_GT OP_LT OP_GE OP_LE
                       OP_LIKE OP_IN OP_ADD OP_SUB OP_MUL OP_DIV OP_MOD OP_CON
                       OP_REP)],
        c_dtypes => [qw(DT_UNKNOWN DT_DEPENDENT DT_BOOL DT_BYTE DT_DATETIME
                        DT_ENUM DT_INT DT_NTPSTAMP DT_PORTLIST DT_REAL DT_STRING
                        DT_PERIOD DT_NUMERIC DT_ADDRESS DT_IPV4 DT_IPV6 DT_MAC
                        DT_EMAIL DT_DURATION DT_IPV4_STR DT_IPV4_HEX DT_IPV4_STR_NET
                        DT_IPV4_STR_NET_MASK DT_IPV6_STR DT_IPV6_HEX DT_IPV6_STR_NET
                        DT_IPV6_STR_NET_MASK DT_REGEXP)],
        c_processor => [qw(CSECTION_LOGER CSECTION_PROCESSOR CSECTION_SCHEDULER CSECTION_COMPONENTS CSECTION_DAEMON
                           CSECTION_STARTUP CSECTION_LOOP CSECTION_SHUTDOWN
                           CKEY_LOG CKEY_PATH CKEY_SIGNALS CKEY_RUNLOG CKEY_CLASS CKEY_PID_FILE CKEY_USER CKEY_GROUP CKEY_INFOREG CKEY_EVENTS
                           STATE_NORMAL STATE_PAUSED)],
        c_hawat => [qw(KEY_CACHE_NODENAMES KEY_CACHE_NODESWS KEY_CACHE_CATEGORIES KEY_CACHE_DETECTORS KEY_CACHE_NODECATEG KEY_CACHE_SWCATEG KEY_CACHE_EVENTLIB
                       HAWAT_MSG_HANDLER HAWAT_ERROR_HANDLER HAWAT_ERROR_HANDLER_R
                       HAWAT_DBI_FIND HAWAT_DBI_LIST HAWAT_DBI_FETCH HAWAT_DBI_CREATE HAWAT_DBI_UPDATE HAWAT_DBI_RENAME HAWAT_DBI_MODIFY HAWAT_DBI_DELETE HAWAT_DBI_REMOVE HAWAT_DBI_SCREATE HAWAT_DBI_SUPDATE HAWAT_DBI_SDELETE HAWAT_DBI_DO
                       HAWAT_ATTR_MID HAWAT_ATTR_ID HAWAT_ATTR_NAME HAWAT_ATTR_ORG HAWAT_ATTR_EMAIL HAWAT_ATTR_ROLES HAWAT_ATTR_GROUPS HAWAT_ATTR_ORGGROUPS HAWAT_ATTR_AFFIL HAWAT_ATTR_QUERIES HAWAT_ATTR_TS
                       HAWAT_ATTR_DESC HAWAT_ATTR_SOURCE HAWAT_ATTR_NETWORKS HAWAT_ATTR_MANAGERS
                       HAWAT_MODEL_ACCESSLOGS HAWAT_MODEL_ALERTS HAWAT_MODEL_CHANGELOGS HAWAT_MODEL_DASHBOARD HAWAT_MODEL_EVENTS HAWAT_MODEL_GROUPS HAWAT_MODEL_LOGS HAWAT_MODEL_PAGINATOR HAWAT_MODEL_QUERY HAWAT_MODEL_REPORTS HAWAT_MODEL_STATISTICS HAWAT_MODEL_USERS HAWAT_MODEL_WHOIS
                       HAWAT_VIEW_HTML HAWAT_VIEW_JSON HAWAT_FORM_AUTOFILL HAWAT_FORM_CANCEL HAWAT_FORM_PREVIEW HAWAT_ACCOUNT_ANONYMOUS
                       HAWAT_FORM HAWAT_FORM_ID HAWAT_FORM_SIMPLE HAWAT_FORM_SAVE HAWAT_FORM_QUERY_NAME
                       HAWAT_FORM_LIMIT HAWAT_FORM_SKIP HAWAT_FORM_SORTBY HAWAT_FORM_SORT HAWAT_FORM_FILTER HAWAT_FORM_IPSRC HAWAT_FORM_IPOR
                       HAWAT_FORM_IPDST HAWAT_FORM_DATEFROM HAWAT_FORM_DATETO HAWAT_FORM_CLASSIFICATION
                       HAWAT_FORM_SELECT_CLASS HAWAT_FORM_DETECTOR HAWAT_FORM_SELECT_DETECTOR HAWAT_FORM_SELECT_GROUP
                       HAWAT_FORM_QUERY HAWAT_FORM_PAGE HAWAT_FORM_PAGINATOR
                       HAWAT_TABLE_BODY HAWAT_TABLE_BODY_USER_QUERY HAWAT_TABLE_BODY_GROUP_QUERY HAWAT_TABLE_QUERY
                       HAWAT_DURATION HAWAT_COUNT HAWAT_COUNT_ALL HAWAT_LAST_FILTER HAWAT_LAST_INDEX HAWAT_FORM_GROUP
                       HAWAT_CHECKED HAWAT_FORWARD_VIEW_HTML HAWAT_FORWARD_VIEW_JSON
                       HAWAT_FORM_FROM HAWAT_FORM_TO HAWAT_FORM_INTERVALS HAWAT_FORM_DATA HAWAT_FORM_ERROR
                       HAWAT_PATH_SOURCE_IP HAWAT_PATH_TARGET_IP HAWAT_PAGE_FIRST_PAGE HAWAT_PAGE_LAST_PAGE
                       HAWAT_PAGE_CURRENT_PAGE HAWAT_PAGE_PREVIOUS_PAGE HAWAT_PAGE_NEXT_PAGE HAWAT_PAGE_MIN
                       HAWAT_PAGE_MAX HAWAT_QUERY_TITLE HAWAT_ID_QUERY HAWAT_ID_GROUPS)],
        func => [qw(&loc_bin &loc_etc &loc_cre &loc_log &loc_run &loc_tmp &loc_var &app_name &json_load &json_save &p)],
    );

    # Everything exported must be in this array
    @EXPORT_OK = ();
    foreach my $tag (keys %EXPORT_TAGS) {
        push(@EXPORT_OK, @{$EXPORT_TAGS{$tag}});
    }

    # Some custom export groups
    $EXPORT_TAGS{all} = [@EXPORT_OK];

    #***************************************************************************
    # Default environment configurations
    #***************************************************************************

    # Setup environment dependent locations
    %LOCATIONS = (
        ENV_SYSTEM() => {
                log => '/var/mentat/log',
                etc => '/etc/mentat',
                cre => '/etc/mentat/.core',
                run => '/var/mentat/run',
                tmp => '/var/mentat/tmp',
                var => '/var/mentat',
            },
        ENV_PACKAGE() => {
                log => '/opt/mentat/var/log',
                etc => '/opt/mentat/etc',
                cre => '/opt/mentat/etc/.core',
                run => '/opt/mentat/var/run',
                tmp => '/opt/mentat/var/tmp',
                var => '/opt/mentat/var',
            },
        ENV_DEVEL() => {
                log => '../spool',
                etc => '../etc',
                cre => '../etc',
                run => '../spool',
                tmp => '../spool',
                var => '../spool',
            },
        ENV_DEVEL_S() => {
                log => '/var/mentat/log',
                etc => '../etc',
                cre => '../etc',
                run => '/var/mentat/run',
                tmp => '/var/mentat/tmp',
                var => '/var/mentat',
            },
        ENV_TEST() => {
                log => './spool',
                etc => './conf',
                cre => './conf',
                run => './spool',
                tmp => './spool',
                var => './spool',
            },
    );

    #***************************************************************************

    # Detect the runtime environment
    detect_runtime_environment();
    #set_runtime_environment(ENV_SYSTEM());
    #set_runtime_environment(ENV_PACKAGE());
    #set_runtime_environment(ENV_DEVEL());
    #set_runtime_environment(ENV_TEST());
}

#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

# No implementation in this module

=pod

=back

=cut

1;
