package Mentat::Message::Formater::TextTemplate;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Mentat::Message::Formater::TextTemplate - Short module description

=head1 SYNOPSIS

 use Mentat::Message::Formater::TextTemplate;

=head1 DESCRIPTION



=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';
use DirHandle;
use File::Basename;

use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#
use Text::Template;

#-- Custom application modules ------------------------------------------------#
use Log::Loger;
use Mentat::Core qw(:func);
use Mentat::Message::Formater;
use Value::Convertor;
use Mentat::Message::Grouper;
use Mentat::Message::Formater::Helper;

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#
use constant T_MASTER_PREFIX => '_master_';
use constant T_SUB_PREFIX    => '_sub_';

use constant DFLT_TEMPLATE   => 'default';

use constant TYPE_SUMMARY => 'summary';
use constant TYPE_UNKNOWN => 'unknown';
use constant TYPE_EXTRA   => 'extra';

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#
my $SERVICE_INSTANCE;

my %TRT_LABELS = (
        TYPE_EXTRA()   => 'label_e',
        TYPE_SUMMARY() => 'label_s',
        TYPE_UNKNOWN() => 'label_s',
    );

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL @ISA);
    $VERSION = '0.1';
    $DEVEL   = 0;
    @ISA     = ('Mentat::Message::Formater');
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item detect_section_ids($) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    :

=cut

sub detect_section_ids($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($message,) = @_;

    my $section_id = $message->path_value('_CESNET/EventTemplate');
    return ($section_id) if $section_id;
    my $categories = $message->path_values('Category');
    return @$categories if $categories and scalar @$categories;
    return ('UnknownCategory');
}

=item format_section($$$$) [PUBLIC

  Usage     : my $section = $formater->format_section($section_id, $data, $report_type, $template_category);
  Purpose   :
  Arguments :
  Returns   : STRING $section
  Throws    : Croaks, if invoked on class

=cut

sub format_section($$$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($section_id, $data, $report_type, $template_category, $section_number) = @_;
    confess "Section ID must be given as argument" unless $section_id;
    confess "Section data must be given as argument" unless $data and ref $data eq 'ARRAY';
    confess "Report type must be given as argument" unless $report_type;

    # Group message according to the IP4 address
    $data = $self->{GROUPER}->group('Source/IP4', $data);

    # Fill in the subtemplate
    my $template_file = ''.T_SUB_PREFIX().$template_category;
    my $section = $self->_fill_template($template_file, {
                                            'T'    => $report_type,
                                            'L'    => $TRT_LABELS{$report_type},
                                            'HL'   => \$self->{HELPER},
                                            'SID'  => $section_id,
                                            'DATA' => $data,
                                            'SNUM' => $section_number,
                                        });
    Log::Loger->debug(__PACKAGE__, "[TT] Done formating '$report_type' section '$section_id' into template '$template_file'") if $DEVEL;
    return $section;
}

#-------------------------------------------------------------------------------

=item format_report($$$;$) [PUBLIC

  Usage     : my $email_message = $formater->format_report($id, $messages, $report);
  Purpose   : Format given messages into summary report
  Arguments : HASH REFERENCE $group - alerts group in hash of hashes
  Returns   : STRING $email_message
  Throws    : Croaks, if invoked on class

=cut

sub format_report($$$;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($report_id, $messages, $report, $template_category) = @_;
    confess "Report ID must be given as argument" unless $report_id;
    confess "Reported messages must be given as argument" unless $messages and ref $messages eq 'ARRAY';
    confess "Report object must be given as argument" unless $report;

    # Set default template category
    $template_category = DFLT_TEMPLATE() unless $template_category;

    my $hash = {'T'  => $report->{type},
                'ID' => $report_id,
                'HL' => \$self->{HELPER},
                'RP' => $report,
            };

    # Divide messages into sections, in which they will be reported
    my $sections = {};
    my (@section_ids, $analyzer_id);
    foreach my $msg (@$messages) {
        $analyzer_id = $msg->path_value('Node[#]/SW');
        $analyzer_id = $msg->path_value('Node[#]/Name') unless $analyzer_id;
        $analyzer_id = 'UnknownAnalyzer' unless $analyzer_id;
        @section_ids = $self->detect_section_ids($msg);
        foreach my $sid (@section_ids) {
            push(@{$sections->{$analyzer_id}->{$sid}}, $msg);
        }
    }

    # Format each section independently
    my $section_body;
    my $section_counter = 0;
    foreach my $anid (sort keys %$sections) {
        foreach my $sid (sort keys %{$sections->{$anid}}) {
            $section_counter += 1;

            # Generate the section body
            $section_body = $self->format_section($sid, $sections->{$anid}->{$sid}, $report->{type}, $template_category, $section_counter);
            # Store the subtemplate to be filled into the master template
            push(@{$hash->{'subtemplates'}}, $section_body);
        }
    }

    # Fill in the master template
    my $master_template = ''.T_MASTER_PREFIX().$template_category;
    return $self->_fill_template($master_template, $hash, 'master');
}

=item stats() [PUBLIC]

  Usage     : my $stats = $formater->stats();
  Purpose   : Get formater statistics
  Arguments : NONE
  Returns   : HASH REFERENCE $stats
  Throws    : Croaks, if invoked on object

=cut

sub stats()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    return $self->{HELPER}->stats() if $self->{HELPER};
    return {};
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Mentat::Message::Formater::TextTemplate instance
#  Arguments :
#  Returns   : Mentat::Message::Formater::TextTemplate reference
#  Throws    : Croaks, if invoked on class
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.
#  See Also  :

sub _init
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($template_dir, $helper) = @_;
    confess "Template directory must be given as argument" unless $template_dir;
    confess "Helper instance must be given as argument" unless blessed($helper);

    $self->{GROUPER}        = Mentat::Message::Grouper->new();
    $self->{TEMPLATE_DIR}   = $template_dir;
    $self->{TEMPLATE_FILES} = $self->_find_templates_files($template_dir);

    $self->{HELPER} = $helper;

    $self->{TEMPLATES} = {};

    return $self;
}

# _find_templates_files($) [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  :
#  See Also  :

sub _find_templates_files($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($template_dir,) = @_;

    my $templates = {};

    my ($dir_handle, $file, $template);
    opendir($dir_handle, $template_dir) or Log::Loger->log_exit(__PACKAGE__, 'EMERG', "[TT] Can't open template folder '$template_dir': $!");
    while (defined($file = readdir($dir_handle))) {
        # Only plain files
        next unless (-f "$template_dir/$file");

        # Untaint file and directory names
        if ($template_dir =~ /(.*)/) { $template_dir = $1; }
        if ($file =~ /(.*)/) { $file = $1; }

        $template = $file; $template =~ s/\.tt$//;
        $templates->{$template} = "$template_dir/$file" ;
        Log::Loger->info(__PACKAGE__, "[TT] Found template '$template'")
    }
    closedir($dir_handle);

    return $templates;
}

# _fill_template($$) [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  :
#  See Also  :

sub _fill_template($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($template, $hash, $master) = @_;

    # Lazy template instantination
    unless ($self->{TEMPLATES}->{$template}) {
        my $template_file = $self->{TEMPLATE_FILES}->{$template};
        foreach my $i (1..3) {
            Log::Loger->debug(__PACKAGE__, "[TT] Creating template manipulation object for template '$template'") if $DEVEL;
            $self->{TEMPLATES}->{$template} = Text::Template->new(TYPE => 'FILE', SOURCE => $template_file, UNTAINT => 1);
            unless ($self->{TEMPLATES}->{$template}) {
                Log::Loger->error(__PACKAGE__, "[TT] Unable to create template manipulation object for template '$template', attempt nr.$i: ".$Text::Template::ERROR) if $DEVEL;
            }
            else {
                last;
            }
        }
    }

    # Fill in and return the subtemplate
    Log::Loger->debug(__PACKAGE__, "[TT] Filling ".($master?'master ':'')."template '$template'") if $DEVEL;
    return $self->{TEMPLATES}->{$template}->fill_in(HASH => $hash);
}

=pod

=back

=cut

1;
