package Mentat::Processor::Engine::Fetcher;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Mentat::Processor::Engine::Fetcher - Mentat message source engine for pickup folder

=head1 SYNOPSIS

 use Mentat::Processor::Engine::Fetcher;

=head1 DESCRIPTION



=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';
use Time::Local;

#use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#
use Log::Loger;
use Mentat::Core qw(:c_general);
use Mentat::Processor::Engine;
use Data::Filter::Factory;
use Mentat::Message::Convertor::Mongo;
use Mentat::Message::Storage::Mongo;
use Mentat::Message::Query::Mongo;

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#
use constant LAST_RUN_TS        => 'last_run_ts';
use constant MAX_INTERVAL_DELTA => 5;
use constant DFLT_LIMIT         => 1000000;

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL @ISA);
    $VERSION = '0.1';
    $DEVEL   = 0;
    @ISA     = ('Mentat::Processor::Engine');
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item eh_process(;$) [PUBLIC]

  Usage     :
  Purpose   : Perform the module processing for given amount of time
  Arguments :
  Returns   :
  Throws    :
  Comments  :
  See Also  :

=cut

sub eh_process(;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($query, $exact) = @_;

    my ($th, $tl) = $self->time_boundaries();
    Log::Loger->notice(__PACKAGE__, "[ENG:Fetcher] Fetching start with time window '".localtime($tl)."' - '".localtime($th)."' [$tl - $th]");

    my ($messages, $count_all) = $self->fetch_messages($query, $exact);
    my $count = (ref $messages)?scalar(@$messages):0;
    Log::Loger->notice(__PACKAGE__, "[ENG:Fetcher] Query found total of $count_all messages, $count fetched for processing");

    if ($count < $count_all) {
        Log::Loger->email(__PACKAGE__, "WARNING: Only $count messages were fetched for processing from total count of $count_all messages");
    }

    my ($result, @args) = $self->execute_event('messages_fetched', $messages, $count_all, $self->time_boundaries());

    return (EC_CONTINUE, $messages, $count_all);
}

=item DESTROY() [PUBLIC]

  Usage     : This method must not be used directly
  Purpose   : Class destructor
  Arguments : None
  Returns   : Nothing
  Throws    : Dies, if not invoked on object instance
  Comments  :
  See Also  :

=cut

sub DESTROY()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    delete $self->{FFACTORY};
    delete $self->{MQUERY};
    delete $self->{MCONVERTOR};
}

=item time_boundaries() [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Croaks, if invoked on object
  Comments  :
  See Also  :

=cut

sub time_boundaries()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    return ($self->{TIME_HIGH}, $self->{TIME_LOW});
}

#-------------------------------------------------------------------------------

=item startup(;$) [PUBLIC]

  Usage     : $instance->startup();
  Purpose   : Callback to allow the component perform arbitrary startup operations
  Arguments : HASH REFERENCE $config - startup configurations [OPTIONAL]
  Returns   : Nothing
  Throws    : Dies, if not invoked on instance or on error

=cut

sub startup(;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($config,) = @_;
    confess "Component startup configurations must be passed via HASH reference" if $config and ref $config ne 'HASH';

    # Check time boundaries
    $self->_check_timing();
}

=item pstate($;$) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Croaks, if invoked on object
  Comments  :
  See Also  :

=cut

sub pstate($;$)
{
    my $self = shift;
    croak ((caller(0))[3] . ": instance method invoked on class") unless blessed($self);
    return $self->component('pstate')->pstate(@_);
}

=item pstate_set($$) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Croaks, if invoked on object
  Comments  :
  See Also  :

=cut

sub pstate_set($$)
{
    my $self = shift;
    croak ((caller(0))[3] . ": instance method invoked on class") unless blessed($self);
    return $self->component('pstate')->pstate_set(@_);
}

=item pstate_del($) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Dies, if not invoked on object instance

=cut

sub pstate_del($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    return $self->component('pstate')->pstate_del(@_);
}

=item pstate_list() [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Dies, if not invoked on object instance

=cut

sub pstate_list()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    return $self->component('pstate')->pstate_list();
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    :
#  Comments  :
#  See Also  :

sub _init
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my %config = @_;

    # Call parent version first
    $self = $self->SUPER::_init(%config);

    die "[ENG:Fetcher] Filter grammar file must be specified under configuration key 'filter_grammar_file'\n" unless exists $config{filter_grammar_file};
    die "[ENG:Fetcher] Filter grammar file '$config{filter_grammar_file}' must be ordinary file\n" unless -f $config{filter_grammar_file};
    die "[ENG:Fetcher] Run interval must be specified under configuration key 'run_interval'\n" unless exists $config{run_interval};
    die "[ENG:Fetcher] Run interval must be greater than 0\n" unless $config{run_interval} > 0;
    die "[ENG:Fetcher] High time boundary must be specified under configuration key 'time_high'\n" unless exists $config{time_high};

    # Instantinate necessary tools
    $self->{FFACTORY}   = Data::Filter::Factory->new_from_file($config{filter_grammar_file});
    $self->{MCONVERTOR} = Mentat::Message::Convertor::Mongo->new();
    $self->{MQUERY}     = Mentat::Message::Query::Mongo->new(convertor => $self->{MCONVERTOR},
                                                             storage => Mentat::Message::Storage::Mongo->new());

    my $current_time = time();

    # Calculate time boundaries
    $self->{RUN_INTERVAL}   = $config{run_interval};
    $self->{LIMIT}          = $config{limit} || DFLT_LIMIT();
    $self->{TIME_HIGH}      = $config{time_high} || $current_time;
    if ($self->{TIME_HIGH} =~ /(.*)/) { $self->{TIME_HIGH} = $1; }
    $self->{TIME_LOW}       = $self->{TIME_HIGH} - $self->{RUN_INTERVAL};
    Log::Loger->debug(__PACKAGE__, "[ENG:Fetcher] Time boundaries: " . localtime($self->{TIME_HIGH}) . " - " . localtime($self->{TIME_LOW}) . " (".$self->{TIME_HIGH}."-".$self->{TIME_LOW}.")");

    # Adjust/round the time boundaries
    if ($config{adjust_time_boundaries} and lc($config{adjust_time_boundaries}) eq 'mod') {
        $self->{TIME_HIGH}  = $self->{TIME_HIGH} - ($self->{TIME_HIGH} % $self->{RUN_INTERVAL});
        $self->{TIME_LOW}   = $self->{TIME_HIGH} - $self->{RUN_INTERVAL};
        Log::Loger->debug(__PACKAGE__, "[ENG:Fetcher] MOD adjusted time boundaries: " . localtime($self->{TIME_HIGH}) . " - " . localtime($self->{TIME_LOW}) . " (".$self->{TIME_HIGH}."-".$self->{TIME_LOW}.")");
    }
    elsif ($config{adjust_time_boundaries} and lc($config{adjust_time_boundaries}) eq '2hrs') {
        # ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)
        my @th = localtime($self->{TIME_HIGH});
        $th[0] = $th[1] = 0;
        $th[2] = $th[2] - ($th[2] % 2);
        $self->{TIME_HIGH} = timelocal(@th);
        $self->{TIME_LOW} = $self->{TIME_HIGH} - 7200;
        Log::Loger->debug(__PACKAGE__, "[ENG:Fetcher] 2HRS Adjusted time boundaries: " . localtime($self->{TIME_HIGH}) . " - " . localtime($self->{TIME_LOW}) . " (".$self->{TIME_HIGH}."-".$self->{TIME_LOW}.")");
    }
    Log::Loger->notice(__PACKAGE__, "[ENG:Fetcher] Option for reuse: [command-name] --time-high ".$self->{TIME_HIGH});

    return $self;
}

# _get_events_emitted(;@) [PROTECTED]
#
#  Usage     : Used as a hook method from template method register()
#  Purpose   : Return list of events emited by this component
#  Arguments : LIST OF ARRAY REFERENCES @lists - event lists from parent classess [OPTIONAL]
#  Returns   : ARRAY REFERENCE $events containing event names as STRINGS
#  Throws    : Dies, if not invoked on instance

sub _get_events_emitted(;@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my @lists = @_;

    return $self->SUPER::_get_events_emitted([

    ], @lists);
}

# _get_events_handled(;@) [PROTECTED]
#
#  Usage     : Used as a hook method from template method register()
#  Purpose   : Return list of events and appropriate handlers for this component
#  Arguments : LIST OF ARRAY REFERENCES @lists - event handler lists from parent classess [OPTIONAL]
#  Returns   : ARRAY REFERENCE $handlers with internal structure [$event, $handler]
#                  STRING $event   - event name
#                  STRING $handler - event handler name
#  Throws    : Dies, if not invoked on instance

sub _get_events_handled(;@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my @lists = @_;

    return $self->SUPER::_get_events_handled([
        ['process', 'eh_process'],
    ], @lists);
}

# _get_components_linked(;@) [PROTECTED]
#
#  Usage     : Used as a hook method from template method register()
#  Purpose   : Return list of required components
#  Arguments : LIST OF ARRAY REFERENCES @lists - linked component lists from parent classess [OPTIONAL]
#  Returns   : ARRAY REFERENCE $components containing STRINGS with following format:
#                  'type'      - request for component of given type and any name
#                  ':name'     - request for component of any type but given name
#                  'type:name' - request for component of given both type and name
#  Throws    : Dies, if not invoked on instance

sub _get_components_linked(;@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my @lists = @_;

    return $self->SUPER::_get_components_linked([
        ['state', 'pstate'],
    ], @lists);
}

# _check_timing() [PROTECTED]
#
#  Usage     : $self->_check_timing();
#  Purpose   : Check procesing time boundaries
#  Arguments : Hash containing application configurations
#  Returns   :
#  Throws    :
#  Comments  :
#  See Also  :

sub _check_timing()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    my $current_time = time();

    # Retrieve the last and save the current LAST RUN TIME
    $self->{LAST_RUN_TS} = $self->pstate(LAST_RUN_TS());
    $self->pstate_set(LAST_RUN_TS(), $current_time);

    # Check the proper interval
    if ($self->{LAST_RUN_TS}) {
        $self->{RUN_INTERVAL_ACT}   = $current_time - $self->{LAST_RUN_TS};
        $self->{RUN_INTERVAL_DELTA} = $self->{RUN_INTERVAL_ACT} - $self->{RUN_INTERVAL};

        if (abs($self->{RUN_INTERVAL_DELTA}) > MAX_INTERVAL_DELTA()) {
            my $how = ($self->{RUN_INTERVAL_DELTA} < 0) ? 'soon' : 'late';
            Log::Loger->warn(__PACKAGE__, "[ENG:Fetcher] Running too $how with interval ".$self->{RUN_INTERVAL_ACT}." seconds after last run at ".$self->{LAST_RUN_TS}." (".abs($self->{RUN_INTERVAL_DELTA})." seconds $how)");
        }
    }
}

#-------------------------------------------------------------------------------

# _ffactory() [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance
#  Comments  :
#  See Also  :

sub _ffactory()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    return $self->{FFACTORY};
}

# _mquery() [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance
#  Comments  :
#  See Also  :

sub _mquery()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    return $self->{MQUERY};
}

=item fetch_messages(;$$) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Croaks, if invoked on object
  Comments  :
  See Also  :

=cut

sub fetch_messages(;$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($query, $exact) = @_;

    # Get the query time boundaries
    my ($th, $tl) = $self->time_boundaries();
    my $time_query = '(_CESNET/StorageTime ge ' . $tl . ') and (_CESNET/StorageTime le ' . $th . ')';

    # Unless the query was given explicitly, use the default query with time boundaries
    unless ($query) { $query = $time_query; }
    # Otherwise append it to the given query (unless the EXACT flag is set)
    else            { $query = ($exact) ? $query : "($time_query) and $query"; }

    # And query the database
    return $self->_mquery->query_msg($self->_ffactory->filter_from_string($query), undef, $self->{LIMIT});
}

=pod

=back

=cut

1;
